@@ -39,8 +39,6 @@ C
      1       hakt2,thij(3),thi(3),zz(5),lwj,yj,hs2,hs,z,cc,spf,
      2       hhommax,hhommin,az1,hfixmax,hnfix,ssij,spmax,
      3       hhomimin,hhomimax
-C!$    integer omp_get_thread_num
-C!$    external omp_get_thread_num
 C   arrays with variable length are organized as
 C   theta(n,dp1)
 C   bi(n,dp2)
@@ -278,6 +276,7 @@ C   sw       array of "smoothed" weights dim(dls,dls) dls=2*(ih+ihw)+1
 C
 C   temporary arrays set for maximum degree 2
 C
+      use omp_lib
       implicit none
       external kldistp,lkern
       double precision kldistp,lkern
@@ -291,8 +290,6 @@ C
      1       hakt2,thij(3),thi(3),zz(5),lwj,yj,hs2,hs,z,cc,spf,hhomi,
      2       hhommax,az1,hfixmax,hnfix
 
-!$    integer omp_get_thread_num
-!$    external omp_get_thread_num
 
 C   arrays with variable length are organized as
 C   theta(n,dp1)
@@ -491,6 +488,7 @@ C   sw       array of "smoothed" weights dim(dls,dls) dls=2*(ih+ihw)+1
 C
 C   temporary arrays set for maximum degree 2
 C
+      use omp_lib
       implicit none
       external kldistp,lkern
       double precision kldistp,lkern
@@ -504,8 +502,6 @@ C
       double precision bii(15),sij,swj(15),swj2(15),swj0(15),swjy(6),
      1       z1,z2,wj,hakt2,thij(6),thi(6),zz(15),lwj,hs2,hs,z,cc,
      2       wjy,spf,hhomi,hhommax,az1,hfixmax,hnfix
-!$      integer omp_get_thread_num
-!$      external omp_get_thread_num
 C   arrays with variable length are organized as
 C   theta(n1,n2,dp1)
 C   bi(n1,n2,dp2)
@@ -767,6 +763,7 @@ C   sw       array of "smoothed" weights dim(dls,dls) dls=2*(ih+ihw)+1
 C
 C   temporary arrays set for maximum degree 2
 C
+      use omp_lib
       implicit none
       external kldistp,lkern
       double precision kldistp,lkern
@@ -780,8 +777,6 @@ C
       double precision bii(15),sij,swj(15),swj2(15),swj0(15),swjy(6),
      1       z1,z2,wj,hakt2,thij(6),thi(6),zz(15),lwj,hs2,hs,z,cc,
      2       wjy,spf,hhomi,hhommax,az1,hfixmax,hnfix
-!$    integer omp_get_thread_num
-!$    external omp_get_thread_num
 C   arrays with variable length are organized as
 C   theta(n1,n2,dp1)
 C   bi(n1,n2,dp2)
